//
//  SeamyView.m
//  seamus
//
//  Created by Wim Lewis on 23Aug07.
//  Copyright 2007 Wim Lewis.
//  You may distribute and/or modify this code under the terms of the GNU General Public License as published by the Free Software Foundation, version 2, or (at your option) any later version.
//

#import "SeamyView.h"
#import "Seamer.h"

@implementation SeamyView

- (id)initWithFrame:(NSRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
    }
    return self;
}

- (void)setImage:(NSImage *)background
{
    [bgImage autorelease];
    bgImage = [background retain];
    mouseRow = mouseCol = -1;
    [self setNeedsDisplay:YES];
}

- (void)setSeamer:(Seamer *)sr
{
    [seamer release];
    seamer = [sr retain];
    mouseRow = mouseCol = -1;
    [self setNeedsDisplay:YES];
}

- (void)drawRect:(NSRect)rect
{
    int rows = seamer->rows;
    int cols = seamer->cols;
    NSRect bs = [self bounds];
    
    // Draw the energy function image in the background
    [bgImage drawInRect:bs fromRect:(NSRect){{0,0},[bgImage size]} operation:NSCompositeSourceOver fraction:1.0];
    
    // Draw the seams computed so far, in red. Use CGContextStrokeLineSegments() for efficient drawing of long polylines.
    [[NSColor redColor] set];
    CGContextRef cg = [[NSGraphicsContext currentContext] graphicsPort];
    CGPoint *ps = malloc(sizeof(*ps) * (2*rows));
    CGContextSetLineWidth(cg, 1.0 /* bs.size.width / cols */);
    
    for(int seamIndex = 0; seamIndex < seamer->seamcount; seamIndex ++) {
        int *aSeam = seamer->seams[seamIndex];
        for(int row = 0; row < rows; row ++) {
            ps[row*2].x = bs.origin.x + ( bs.size.width / cols ) * aSeam[row];
            ps[row*2].y = bs.size.height - ( bs.origin.y + ( bs.size.height / rows ) * row );
            if (row > 0)
                ps[row*2-1] = ps[row*2];
        }
        CGContextStrokeLineSegments(cg,ps,rows*2-2);
    }
    
    // If there is a recent mouse-click/drag location, compute a least-cost path from that point and display it in yellow.
    if (mouseRow > 0 && mouseCol > 0) {
        short **lastUpmap = seamer->last_upmap;
        int row = mouseRow, col = mouseCol;
        int pointIndex = 0;
        while(row >= 0) {
            ps[pointIndex].x = bs.origin.x + ( bs.size.width / cols ) * col;
            ps[pointIndex].y = bs.size.height - ( bs.origin.y + ( bs.size.height / rows ) * row );
            pointIndex ++;
            if(pointIndex > 1) {
                ps[pointIndex] = ps[pointIndex-1];
                pointIndex ++;
            }
            
            col = lastUpmap[row][col];
            if (col < 0)
                break;
            row --;
        }
        if (pointIndex > 3) {
            [[NSColor yellowColor] set];
            CGContextStrokeLineSegments(cg,ps,pointIndex-1);
        }
    }
    free(ps);
}

- (void)setTestPoint:(NSPoint)p
{
    // Convert point 'p' to image pixel coordinates and stash it in mouseRow/mouseCol.
    
    int rows = seamer->rows;
    int cols = seamer->cols;
    NSRect bs = [self bounds];
    
    p.x = (p.x - bs.origin.x) * ( cols / bs.size.width );
    p.y = (p.y - bs.origin.y) * ( rows / bs.size.height );
    p.y = rows - p.y;
    
    int newRow = p.y;
    int newCol = p.x;
    if (newRow < 0 || newRow >= rows || newCol < 0 || newCol >= cols) {
        newRow = -1;
        newCol = -1;
    }
    
    if (newRow != mouseRow || newCol != mouseCol) {
        mouseRow = newRow;
        mouseCol = newCol;
        [self setNeedsDisplay:YES];
    }
}

- (void)mouseDown:(NSEvent *)theEvent;
{
    NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    [self setTestPoint:p];
}

- (void)mouseDragged:(NSEvent *)theEvent;
{
    NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
    [self setTestPoint:p];
}


@end
